% cmaint.m: function to convert expected average *lifetime* maintenance costs into the current expected maintenance
%           costs at the current machine age. Note that emaint function computes (from the maintenance cost regressions)
%           the expected average monthly maintenance costs over the full life of the machine (until it is sold). We want
%           the current month expected maintenance costs, so we compute it below as the difference of two moving averages.
%
%           Copyright, all rights reserved, John Rust, Georgetown University, John McClelland, ARA June, 2018
 
   function [cmv]=cmaint(age,ms,month);

   global machine make company region log_maint_var log_maint_beta fc;

   if (age == 0);

       cmv=0;

   else;

       cmv=age*emaint(age,ms,month)-(age-1)*emaint(age-1,ms,month); 

   end;

   if (fc);      % add on any "fixed costs" that are independent of age 
    cmv=cmv+fc;
   end;
