% dpsolve.m: program to solve the dynamic programming problem (either by successive approximations or policy iteration)
%            Copyright, all rights reserved, John Rust, Georgetown University, Jhn McCllelland, ARA June 2018

clear;

setup;      % initialize parameters

global oec machine make company region logit_b sales_logit_b utilization_logit_b log_maint_beta log_maint_var log_rental_beta log_rental_var;
global log_sales_beta log_sales_var log_oec_beta log_oec_var log_oec_sales_beta log_oec_sales_var mtp_pc pf;
global machine_model_names machine_names company_names region_names machine_model pms;
global purchase_price resale_price;

month_names={'January','February','March','April','May','June','July','August','September','October','November','December'};

pms=0;                  % set to 1 to use the results from the proportional (restricted) specification 
                        %          for prices of machines where macro states are restricted to affect
                        %          price function proportionately for all ages.
                        % set to 0 for unrestricted specification where the macro states can interact
                        %          with depreciation, resulting in non-proporational shifts in the
                        %          prices of machines of different ages in different macro states

loopall=0;              % set to 1 do loop through all machine, make and company combinations to solve the DP
                        % problem for a given region
                        % set to 0 to solve model for only a single machine/make/company/region combination
                        % specified below

machine=1;  % choose type of machine to solve (1=Excavator,2=Hi-Reach Forklift,3=Skid Steer,4=Scissors Lift,5=Telescopic boom)

make=4;     % choose a model of machine to solve
            % $make_codes{'Telescopic Boom'}{'M1'}=1;
            % $make_codes{'Telescopic Boom'}{'M2'}=2;
            % $make_codes{'Telescopic Boom'}{'M3'}=3;
            % $make_codes{'Hi-Reach Forklift'}{'M1'}=1;
            % $make_codes{'Hi-Reach Forklift'}{'M2'}=2; 
            % $make_codes{'Hi-Reach Forklift'}{'M3'}=3; 
            % $make_codes{'Hi-Reach Forklift'}{'M4'}=4;
            % $make_codes{'Hi-Reach Forklift'}{'M5'}=5;
            % $make_codes{'Hi-Reach Forklift'}{'M6'}=6;
            % $make_codes{'Scissor Lift'}{'M1'}=1; 
            % $make_codes{'Scissor Lift'}{'M2'}=2;
            % $make_codes{'Scissor Lift'}{'M3'}=3; 
            % $make_codes{'Excavator'}{'M1'}=1;
            % $make_codes{'Excavator'}{'M2'}=2;
            % $make_codes{'Excavator'}{'M3'}=3;
            % $make_codes{'Excavator'}{'M4'}=4; 
            % $make_codes{'Excavator'}{'M5'}=5; 
            % $make_codes{'Skid Steer'}{'M1'}=1; 
            % $make_codes{'Skid Steer'}{'M2'}=2;
company=1;  % company to solve for 
            % $company_codes{'A'}=1;
            % $company_codes{'B'}=2;
            % $company_codes{'C'}=3;
            % $company_codes{'D'}=4;
            % $company_codes{'E'}=5;
            % $company_codes{'F'}=6;
region=6;   % region to solve for
            % $region_codes{'Northwest'}=1;
            % $region_codes{'Southeast'}=2;
            % $region_codes{'West'}=3;
            % $region_codes{'Northeast'}=4;
            % $region_codes{'Midwest'}=5;
            % $region_codes{'Southwest'}=6;

machine_names={'Excavator','Hi-Reach Forklift','Skid Steer','Scissor Lift','Telescopic Boom'};
company_names={'A','B','C','D','E','F'};
machine_model_names=[];
mmn=struct;
mmn.name={'M1','M2','M3','M4','M5'};
machine_model_names=[machine_model_names; mmn];
mmn.name={'M1','M2','M3','M4','M5','M6'};
machine_model_names=[machine_model_names; mmn];
mmn.name={'M1','M2'};
machine_model_names=[machine_model_names; mmn];
mmn.name={'M1','M2','M3'};
machine_model_names=[machine_model_names; mmn];
mmn.name={'M1','M2','M3'};
machine_model_names=[machine_model_names; mmn];
region_names={'Northwest','Southeast','West','Northeast','Midwest','Southwest'};

load('mat/logit_b');

if (loopall);
   machine0=1;
   machine1=5;
else;
   machine0=machine;
   machine1=machine;
end;

for machine=machine0:machine1; 
   if (loopall);
     make0=1;
     make1=size(machine_model_names(machine).name,2);
     company0=1;
     company1=6; 
   else;
     make0=make;
     make1=make;
     company0=company;
     company1=company;
   end;
   for make=make0:make1;
       for company=company0:company1;

if (machine == 1);
   log_maint_beta=load('mat/excav_log_maint_beta');
   log_maint_var=load('mat/excav_log_maint_var');
   log_rental_beta=load('mat/excav_log_rental_beta');
   log_rental_var=load('mat/excav_log_rental_var');
   log_sales_beta=load('mat/excav_log_sales_beta');
   log_sales_var=load('mat/excav_log_sales_var');
   log_oec_beta=load('mat/excav_log_oec_beta');
   log_oec_var=load('mat/excav_log_oec_var');
   if (pms);
     log_oec_sales_beta=load('mat/pms/excav_log_oec_sales_beta');
     log_oec_sales_var=load('mat/pms/excav_log_oec_sales_var');
   else;
     log_oec_sales_beta=load('mat/excav_log_oec_sales_beta');
     log_oec_sales_var=load('mat/excav_log_oec_sales_var');
   end;
   sales_logit_b=load('mat/excavator_sales_logit_b');
   utilization_logit_b=load('mat/excavator_capacity_utilization_logit_b');
elseif (machine == 2);
   log_maint_beta=load('mat/hi_reach_forklift_log_maint_beta');
   log_maint_var=load('mat/hi_reach_forklift_log_maint_var');
   log_rental_beta=load('mat/hi_reach_forklift_log_rental_beta');
   log_rental_var=load('mat/hi_reach_forklift_log_rental_var');
   log_sales_beta=load('mat/hi_reach_forklift_log_sales_beta');
   log_sales_var=load('mat/hi_reach_forklift_log_sales_var');
   log_oec_beta=load('mat/hi_reach_forklift_log_oec_beta');
   log_oec_var=load('mat/hi_reach_forklift_log_oec_var');
   if (pms);
     log_oec_sales_beta=load('mat/pms/hi_reach_forklift_log_oec_sales_beta');
     log_oec_sales_var=load('mat/pms/hi_reach_forklift_log_oec_sales_var');
   else;
     log_oec_sales_beta=load('mat/hi_reach_forklift_log_oec_sales_beta');
     log_oec_sales_var=load('mat/hi_reach_forklift_log_oec_sales_var');
   end;
   sales_logit_b=load('mat/hi_reach_forklift_sales_logit_b');
   utilization_logit_b=load('mat/hi_reach_forklift_capacity_utilization_logit_b');
elseif (machine == 3);
   log_maint_beta=load('mat/skid_steer_log_maint_beta');
   log_maint_var=load('mat/skid_steer_log_maint_var');
   log_rental_beta=load('mat/skid_steer_log_rental_beta');
   log_rental_var=load('mat/skid_steer_log_rental_var');
   log_sales_beta=load('mat/skid_steer_log_sales_beta');
   log_sales_var=load('mat/skid_steer_log_sales_var');
   log_oec_beta=load('mat/skid_steer_log_oec_beta');
   log_oec_var=load('mat/skid_steer_log_oec_var');
   if (pms);
     log_oec_sales_beta=load('mat/pms/skid_steer_log_oec_sales_beta');
     log_oec_sales_var=load('mat/pms/skid_steer_log_oec_sales_var');
   else;
     log_oec_sales_beta=load('mat/skid_steer_log_oec_sales_beta');
     log_oec_sales_var=load('mat/skid_steer_log_oec_sales_var');
   end;
   sales_logit_b=load('mat/skid_steer_sales_logit_b');
   utilization_logit_b=load('mat/skid_steer_capacity_utilization_logit_b');
elseif (machine == 4);
   log_maint_beta=load('mat/scissor_lift_log_maint_beta');
   log_maint_var=load('mat/scissor_lift_log_maint_var');
   log_rental_beta=load('mat/scissor_lift_log_rental_beta');
   log_rental_var=load('mat/scissor_lift_log_rental_var');
   log_sales_beta=load('mat/scissor_lift_log_sales_beta');
   log_sales_var=load('mat/scissor_lift_log_sales_var');
   log_oec_beta=load('mat/scissor_lift_log_oec_beta');
   log_oec_var=load('mat/scissor_lift_log_oec_var');
   if (pms);
     log_oec_sales_beta=load('mat/pms/scissor_lift_log_oec_sales_beta');
     log_oec_sales_var=load('mat/pms/scissor_lift_log_oec_sales_var');
   else;
     log_oec_sales_beta=load('mat/scissor_lift_log_oec_sales_beta');
     log_oec_sales_var=load('mat/scissor_lift_log_oec_sales_var');
   end;
   sales_logit_b=load('mat/scissor_lift_sales_logit_b');
   utilization_logit_b=load('mat/scissor_lift_capacity_utilization_logit_b');
else;
   log_maint_beta=load('mat/telescopic_boom_log_maint_beta');
   log_maint_var=load('mat/telescopic_boom_log_maint_var');
   log_rental_beta=load('mat/telescopic_boom_log_rental_beta');
   log_rental_var=load('mat/telescopic_boom_log_rental_var');
   log_sales_beta=load('mat/telescopic_boom_log_sales_beta');
   log_sales_var=load('mat/telescopic_boom_log_sales_var');
   log_oec_beta=load('mat/telescopic_boom_log_oec_beta');
   log_oec_var=load('mat/telescopic_boom_log_oec_var');
   if (pms);
     log_oec_sales_beta=load('mat/pms/telescopic_boom_log_oec_sales_beta');
     log_oec_sales_var=load('mat/pms/telescopic_boom_log_oec_sales_var');
   else;
     log_oec_sales_beta=load('mat/telescopic_boom_log_oec_sales_beta');
     log_oec_sales_var=load('mat/telescopic_boom_log_oec_sales_var');
   end;
   sales_logit_b=load('mat/telescopic_boom_sales_logit_b');
   utilization_logit_b=load('mat/telescopic_boom_capacity_utilization_logit_b');
end;

log_maint_var=log_maint_var.var;
log_maint_beta=log_maint_beta.b;
log_rental_var=log_rental_var.var;
log_rental_beta=log_rental_beta.b;
log_sales_var=log_sales_var.var;
log_sales_beta=log_sales_beta.b;
log_oec_sales_var=log_oec_sales_var.var;
log_oec_sales_beta=log_oec_sales_beta.b;
log_oec_var=log_oec_var.var;
log_oec_beta=log_oec_beta.b;
sales_logit_b=sales_logit_b.sales_logit_b;
utilization_logit_b=utilization_logit_b.utilization_logit_b;

% calculate average purchase price of a new machine by month (averaging over the invariant probability
% of different macro states in each month)

erms=ergodic_macro_states;  % calculate the invariant probability for the macro states including seasonal effects,
                            % so erms is a probability distribution of size nm*12  with macro states grouped together
                            % ermms(1),....,erms(nm) is the ergodic probability probability for January,
                            % ermms(nm+1),....,erms(2*nm) is the ergodic probability probability for February, and so forth

purchase_price=0;
resale_price=0;
for i=1:12;
   for j=1:nm;
     purchase_price=purchase_price+oecsales(0,j,i)*erms((i-1)*nm+j);
     resale_price=resale_price+oecsales(accounting_life,j,i)*erms((i-1)*nm+j);
   end;
end;

solve_dp=1;

if (solve_dp);

% Solve DP problem using policy iteration (this version uses dense linear algebra 
%                                          and does not attempt to exploit the sparsity structure
%                                          of the transition probability matrix tpm using the
%                                          cyclic inversion algorithm of Paarsch and Rust, 

  if (pms);
   fname=sprintf('results_pms_%s_%s_%s_%s_rr%i_notax%i_%s',...
   machine_names{machine},machine_model_names(machine).name{make},company_names{company},region_names{region},rr,notax,tax_regime);
  else;
   fname=sprintf('results_%s_%s_%s_%s_rr%i_notax%i_%s',...
   machine_names{machine},machine_model_names(machine).name{make},company_names{company},region_names{region},rr,notax,tax_regime);
  end;
  if (~exist(fname));
    mkdir(fname);
  else;
    system(sprintf('touch "%s"',fname));
  end;

  diary(sprintf('%s/dpsolve.out',fname));

  fprintf('Dynamic programming solution for optimal replacement policy for %s (%s) for company %s in %s\n\n',...
  machine_names{machine},machine_model_names(machine).name{make},company_names{company},region_names{region});
  if (notax);
      fprintf('Taxes are not modeled in the DP solution\n');
  else;
      fprintf('Taxes are modeled and accounted for in the DP solution under the %s tax regime (tax rate %g)\n',tax_regime,tax_rate);
  end;
  if (rr > 0);
     fprintf('Replacement strategy is restricted to replacing an old machine with a brand new one\n');
  elseif (rr == 0);
     fprintf('Replacement strategy is restricted to replacing an old machine with one that is less than or equal to it in age\n');
  else;
     fprintf('Replacement strategy is restricted to replacing an old machine with one that is no more than %i momths old\n',-rr);
  end;

  fprintf('\n\n');

  mtp_pc=zeros(12*nm,nm);

  for month=1:12;
     for ms=1:nm;
        i=ms+nm*(month-1);
        mtp_pc(i,:)=mtp(month,ms);
     end;
  end;

  i = 0;
  chgv = 1.e99;

  while (chgv > tol && i<maxit);

    i = i+1;

    [vl,pf] = policy_improvement(v);  % Policy improvement step

    chgv1 = max(abs(vl-v));

    vl = (eye(n)-bet*p)\pf;             % Policy valuation step

    chgv = max(abs(vl-v));

    fprintf(' Step %2i, change in v = %12.9f  (change from 1 contraction step %12.9f)\n',i,chgv,chgv1);

    v = vl;

  end;

  if (i < maxit);

   fprintf('\nPolicy iteration converged in %2i steps, change in v = %7.4f\n',i,chgv);

  else;

   fprintf('\nReached iteration limit: may not have converged, change in v= %7.4f\n',chgv);

  end;

  [vsq,pfsq]=policy_valuation;
  [vsq_ss,pfsq_ss]=policy_valuation_single_cycle;
  [v_ss,pf_ss]=policy_valuation_opt_single_cycle;

  cd(fname);

  fname=sprintf('v_%s_%s_%s_%s_rr%i',machine_names{machine},machine_model_names(machine).name{make},company_names{company},region_names{region},rr);
  save(fname,'v');
  fname=sprintf('d_%s_%s_%s_%s_rr%i',machine_names{machine},machine_model_names(machine).name{make},company_names{company},region_names{region},rr);
  save(fname,'d');
  fname=sprintf('pf_%s_%s_%s_%s_rr%i',machine_names{machine},machine_model_names(machine).name{make},company_names{company},region_names{region},rr);
  save(fname,'pf');

  fname=sprintf('vsq_%s_%s_%s_%s',machine_names{machine},machine_model_names(machine).name{make},company_names{company},region_names{region});
  save(fname,'vsq');
  fname=sprintf('pfsq_%s_%s_%s_%s',machine_names{machine},machine_model_names(machine).name{make},company_names{company},region_names{region});
  save(fname,'pfsq');

  fname=sprintf('vsq_ss_%s_%s_%s_%s',machine_names{machine},machine_model_names(machine).name{make},company_names{company},region_names{region});
  save(fname,'vsq_ss');
  fname=sprintf('pfsq_ss_%s_%s_%s_%s',machine_names{machine},machine_model_names(machine).name{make},company_names{company},region_names{region});
  save(fname,'pfsq_ss');

  fname=sprintf('v_ss_%s_%s_%s_%s',machine_names{machine},machine_model_names(machine).name{make},company_names{company},region_names{region});
  save(fname,'v_ss');
  fname=sprintf('pf_ss_%s_%s_%s_%s',machine_names{machine},machine_model_names(machine).name{make},company_names{company},region_names{region});
  save(fname,'pf_ss');

  cd ..;

end;

fprintf('percentage gain: optimal policy relative to status quo, 1 month old machine\n');
av_vf=0;
av_vfsq=0;
av_oec=0;
for i=1:12;
    av_vf=av_vf+vf(1,1,i)+vf(1,2,i)+vf(1,3,i);
    av_vfsq=av_vfsq+vfsq(1,1,i)+vfsq(1,2,i)+vfsq(1,3,i);
    av_oec=av_oec+oecsales(0,1,i)+oecsales(0,2,i)+oecsales(0,3,i);
    fprintf('%s %g  %g  %g\n',month_names{i},(vf(1,1,i)-vfsq(1,1,i))/vfsq(1,1,i),(vf(1,2,i)-vfsq(1,2,i))/vfsq(1,2,i),(vf(1,3,i)-vfsq(1,3,i))/vfsq(1,3,i));
end;
   av_vf=av_vf/36;
   av_vfsq=av_vfsq/36;
   av_oec=av_oec/36;
   fprintf('\nSummary:  optimal policy (month 1, average all macro states and months)\n');
   fprintf('Optimal policy   :  average value: %g    monthly: %g    percentage gain: %g\n',av_vf,(1-bet)*av_vf,100*(av_vf-av_vfsq)/av_vfsq);
   fprintf('Status quo policy:  average value: %g    monthly: %g\n',av_vfsq,(1-bet)*av_vfsq);
   fprintf('\nValue to OEC ratio:  optimal: %g  status quo %g\n',av_vf/av_oec,av_vfsq/av_oec);

fprintf('percentage gain: optimal policy relative to status quo, 80 month old machine\n');
av_vf=0;
av_vfsq=0;
av_oec=0;
for i=1:12;
    av_vf=av_vf+vf(80,1,i)+vf(80,2,i)+vf(80,3,i);
    av_vfsq=av_vfsq+vfsq(80,1,i)+vfsq(80,2,i)+vfsq(80,3,i);
    av_oec=av_oec+oecsales(0,1,i)+oecsales(0,2,i)+oecsales(0,3,i);
    fprintf('%s %g  %g  %g\n',month_names{i},(vf(80,1,i)-vfsq(80,1,i))/vfsq(80,1,i),(vf(80,2,i)-vfsq(80,2,i))/vfsq(80,2,i),(vf(80,3,i)-vfsq(80,3,i))/vfsq(80,3,i));
end;
   av_vf=av_vf/36;
   av_vfsq=av_vfsq/36;
   av_oec=av_oec/36;
   fprintf('\nSummary:  optimal policy (month 80, average all macro states and months)\n');
   fprintf('Optimal policy   :  average value: %g    monthly: %g    percentage gain: %g\n',av_vf,(1-bet)*av_vf,100*(av_vf-av_vfsq)/av_vfsq);
   fprintf('Status quo policy:  average value: %g    monthly: %g\n',av_vfsq,(1-bet)*av_vfsq);
   fprintf('\nValue to OEC ratio:  optimal: %g  status quo %g\n',av_vf/av_oec,av_vfsq/av_oec);

  diary off;

    end;
  end;
end;

plotresults(12);   % argument is the base month for plotting optimal decision rules and value functions, can be an integer from 1 to 12
