% interest.m  program to calculate interest and principal payments over an assumed lifetime on an acquired machine
%             Copyright, all rights reserved, John Rust, Georgetown University, John McClelland, ARA, June 2018

function [ipf]=interest(n,r);

% n is the lifetime of the loan in years (monthly servicing)
% r is the annual interest rate

% returns: a martix of dimension 12*n x 4 with the following 4 columns
%          showing, by month over the duration of the loan, the following quantities
%          p: unpaid principal balance remaining
%          k: flat monthly principal and interest payment, calculated at annual interest rate r
%          int: interest payment component of the flat monthly payment k
%          pp:  principal repayment component of the flat monthly payment k


    bet=1/(1+r/12);
   
    n1=n+1;
    k=(1-bet)/(bet-bet^n1); 

    p=1;
    total_payments=0;
    total_interest=0;
    total_pp=0;

    ipf=[];

    for i=1:n;

      int=p*r/12;
      pp=(k-int);
      %total_payments=total_payments+k;
      %total_interest=total_interest+int;
      %total_pp=total_pp+pp;

      ipf=[ipf; p k int pp];

      %fprintf('%i principal: %g total payment %g interest %i principal payment %g\n',i,p,k,int,pp);

      p=p-pp;
  
    end;  

    %fprintf('Total payments: %g  total principal payments %g total interest payments %g\n',total_payments,total_pp,total_interest);


