% plotresults.m: plots the value function, profit function and decision rule
%                from the solution of the DP problem. 
%                Copyright, all rights reserved, John Rust, Georgetown University, John McClelland, ARA June 2018

function plotresults(month);

global v vsq pf d nm na machine company make region trc;
global machine_names company_names region_names machine_model_names;

month_names={'January','February','March','April','May','June','July','August','September','October','November','December'};
mon_names=['Jan';'Feb';'Mar';'Apr';'May';'Jun';'Jul';'Aug';'Sep';'Oct';'Nov';'Dec'];

x=(1:na);
clf;

y=zeros(na,nm);

for i=1:na;
   y(i,1)=dr(i,1,month);
   y(i,2)=dr(i,2,month);
   y(i,3)=dr(i,3,month);
end;
  figure(1);
  hold on;
  plot(x,y(:,3),'-r','Linewidth',2);
  plot(x,y(:,2),'-k','Linewidth',2);
  plot(x,y(:,1),'-b','Linewidth',2);
  legend('Boom','Normal','Bust','Location','NorthWest');
  xlabel('Age of machine (months)');
  ylabel('Optimal decision (0=trade for new machine)');
  %title({sprintf('Optimal decision rules for %s (%s) in %s',machine_names{machine},machine_model_names(machine).name{make},month_names{month}),...
  %sprintf('Company %s in %s',company_names{company},region_names{region})});
  title({sprintf('Optimal decision rules for make %i %s in %s',make,machine_names{machine},month_names{month}),...
  sprintf('Company %s in region %i',company_names{company},region)});
  %title(sprintf('Optimal decision rules for %s (%s) in %s',machine_names{machine},machine_model_names(machine).name{make},month_names{month}));
  set(gca,'XTick',(0:20:na));
  hold off;

for i=1:na;
   if (dr(i,1,month) == i);
     y(i,1)=0;
   else;
     y(i,1)=1;
   end;
   if (dr(i,2,month) == i);
     y(i,2)=0;
   else;
     y(i,2)=1;
   end;
   if (dr(i,3,month) == i);
     y(i,3)=0;
   else;
     y(i,3)=1;
   end;
end;
  figure(2);
  hold on;
  plot(x,y(:,3),'-r','Linewidth',2);
  plot(x,y(:,2),'-k','Linewidth',2);
  plot(x,y(:,1),'-b','Linewidth',2);
  legend('Boom','Normal','Bust','Location','NorthWest');
  xlabel('Age of machine (months)');
  ylabel('Probability of replacing machine');
  %title({sprintf('Replacement probability for %s (%s) in %s',machine_names{machine},machine_model_names(machine).name{make},month_names{month}),...
  %sprintf('Company %s in %s',company_names{company},region_names{region})});
  title({sprintf('Replacement probability for make %i %s in %s',make,machine_names{machine},month_names{month}),...
  sprintf('Company %s in region %i',company_names{company},region)});
  set(gca,'XTick',(0:20:na));
  hold off;

  yact=zeros(na,3);
  for i=1:na;
     yact(i,1)=salep(i,1,month);
     yact(i,2)=salep(i,2,month);
     yact(i,3)=salep(i,3,month);
  end;

  figure(3);
  hold on;
  plot(x,y(:,3),'-r','Linewidth',2);
  plot(x,y(:,2),'-k','Linewidth',2);
  plot(x,y(:,1),'-b','Linewidth',2);
  plot(x,yact(:,3),'--r','Linewidth',2);
  plot(x,yact(:,2),'--k','Linewidth',2);
  plot(x,yact(:,1),'--b','Linewidth',2);
  legend('Boom,opt','Normal,opt','Bust,opt','Boom,act','Normal,act','Bust,act','Location','SouthEast');
  xlabel('Age of machine (months)');
  ylabel('Probability of replacing machine');
  %title({sprintf('Replacement probabilities for %s (%s) in %s',machine_names{machine},machine_model_names(machine).name{make},month_names{month}),...
  %sprintf('Company %s in %s',company_names{company},region_names{region})});
  title({sprintf('Replacement probabilities for make %i %s in %s',make,machine_names{machine},month_names{month}),...
  sprintf('Company %s in region %i',company_names{company},region)});
  set(gca,'XTick',(0:20:na));
  hold off;

for i=1:na;
   y(i,1)=vf(i,1,month);
   y(i,2)=vf(i,2,month);
   y(i,3)=vf(i,3,month);
   yact(i,1)=vfsq(i,1,month);
   yact(i,2)=vfsq(i,2,month);
   yact(i,3)=vfsq(i,3,month);
end;
  figure(4);
  hold on;
  plot(x,y(:,3),'-r','Linewidth',2);
  plot(x,y(:,2),'-k','Linewidth',2);
  plot(x,y(:,1),'-b','Linewidth',2);
  plot(x,yact(:,3),'--r','Linewidth',2);
  plot(x,yact(:,2),'--k','Linewidth',2);
  plot(x,yact(:,1),'--b','Linewidth',2);
  legend('Boom,opt','Normal,opt','Bust,opt','Boom,act','Normal,act','Bust,act','Location','NorthEast');
  xlabel('Age of machine (months)');
  ylabel('Expected present value of machine');
  %title({sprintf('Value functions for %s (%s) in %s',machine_names{machine},machine_model_names(machine).name{make},month_names{month}),...
  %sprintf('Company %s in %s',company_names{company},region_names{region})});
  title({sprintf('Value functions for make %i %s in %s',make,machine_names{machine},month_names{month}),...
  sprintf('Company %s in region %i',company_names{company},region)});
  set(gca,'XTick',(0:20:na));
  hold off;

   rc=zeros(na,3);

for i=1:na;
   y(i,1)=profit(i,1,month);
   y(i,2)=profit(i,2,month);
   y(i,3)=profit(i,3,month);
   rc(i,1)=oecsales(0,1,month)-oecsales(i,1,month)+trc;
   rc(i,2)=oecsales(0,2,month)-oecsales(i,2,month)+trc;
   rc(i,3)=oecsales(0,3,month)-oecsales(i,3,month)+trc;
end;
  figure(5);
  hold on;
  plot(x,y(:,3),'-r','Linewidth',2);
  plot(x,y(:,2),'-k','Linewidth',2);
  plot(x,y(:,1),'-b','Linewidth',2);
  legend('Boom','Normal','Bust','Location','SouthWest');
  xlabel('Age of machine (months)');
  ylabel('Current month net cash flow from machine');
  %title({sprintf('Profit functions for %s (%s) in %s',machine_names{machine},machine_model_names(machine).name{make},month_names{month}),...
  %sprintf('Company %s in %s',company_names{company},region_names{region})});
  title({sprintf('Profit functions for make %i %s in %s',make,machine_names{machine},month_names{month}),...
  sprintf('Company %s in region %i',company_names{company},region)});
  set(gca,'XTick',(0:20:na));
  hold off;

  figure(6);
  hold on;
  plot(x,rc(:,3),'k-','Linewidth',2);
  plot(x,rc(:,2),'k--','Linewidth',2);
  plot(x,rc(:,1),'k:','Linewidth',2);
  legend('Boom','Normal','Bust','Location','SouthEast');
  xlabel('Age of machine (months)');
  ylabel('Replacement costs');
  %title({sprintf('Profit functions for %s (%s) in %s',machine_names{machine},machine_model_names(machine).name{make},month_names{month}),...
  %sprintf('Company %s in %s',company_names{company},region_names{region})});
  title({sprintf('Replacement costs for make %i %s in %s',make,machine_names{machine},month_names{month}),...
  sprintf('Company %s in region %i',company_names{company},region)});
  set(gca,'XTick',(0:20:na));
  hold off;

  rt=na*ones(12,3);
for month=1:12;
for i=na:-1:1;
   if (~(dr(i,1,month) == i));
       rt(month,1)=i;
   end;
   if (~(dr(i,2,month) == i));
       rt(month,2)=i;
   end;
   if (~(dr(i,3,month) == i));
       rt(month,3)=i;
   end;
end;
end;
  
  x=(1:12);
  figure(7);
  hold on;
  plot(x,rt(:,3),'-r','Linewidth',2);
  plot(x,rt(:,2),'-k','Linewidth',2);
  plot(x,rt(:,1),'-b','Linewidth',2);
  legend('Boom','Normal','Bust','Location','NorthEast');
  xlabel('Month of year');
  ylabel('Optimal replacement threshold (months)');
  %title({sprintf('Optimal replacement thresholds for %s (%s)',...
  %machine_names{machine},machine_model_names(machine).name{make}),...
  %sprintf('Company %s in %s',company_names{company},region_names{region})});
  title({sprintf('Optimal replacement thresholds for make %i %s',make,machine_names{machine}),...
  sprintf('Company %s in region %i',company_names{company},region)});
  axis('tight');
  set(gca,'XTick',[1:12])
  set(gca,'XTickLabel',mon_names);
  hold off;

