% policy_valuation.m: policy valuation for the firms' status quo replacement policies
%                     Copyright, all rights reserved John Rust, Georgetown University, John McClelland, ARA  June 2018
%
% Outputs:
%
%    vsq:  value of the status quo policy (predicted by the firm's replacement policy given
%          by salep.m)
%

function [vsq,pf]=policy_valuation;

global vl d p n na nm bet oec trc trd rr mtp_pc pr;
global notax accounting_life loan_duration borrowing_rate tax_rate debt_financing interest_deductible ipf book_value nodepreciation;
global erms purchase_price resale_price;

pf=zeros(n,1);
mstp=zeros(nm,nm);
p=0*p;

for month=1:12;       % loop over month of the year

  i1=nm*(month-1)+1;
  i2=i1+nm-1;
  tpm=mtp_pc(i1:i2,:);        
  monthp=nextmonth(month);

  for a=1:na;         % loop over machine ages

    if (a == na);
       ap=na;
    else;
       ap=a+1;
    end;

    for ms=1:nm;      % loop over macro state

        interest_payment=0;
        debt_payment=0;
        depr=0;
        tax=0;

        pfkeep=erentrev(a,ms,month)-cmaint(a,ms,month);

        if (debt_financing);
          if (a > loan_duration);    
            interest_payment=0;
            debt_payment=0;
          else;
            interest_payment=debt_financing*purchase_price*ipf(a,3);
            debt_payment=debt_financing*purchase_price*ipf(a,2);
          end;
        else;
            interest_payment=0;
            debt_payment=0;
        end;

        if (~notax);
          if (~nodepreciation);
              if (a == 1);
                depr=(purchase_price-resale_price)*(1-book_value(a));
              elseif (a < accounting_life);
                depr=(purchase_price-resale_price)*(book_value(a-1)-book_value(a));
              else;
                depr=0;
              end;
          end;

          % calculate EBT (Earnings before taxes)

          if (interest_deductible);
            accounting_profit=pfkeep-interest_payment-depr;
          else;
            accounting_profit=pfkeep-depr;
          end;
          % compute tax
          tax=tax_rate*accounting_profit;
        else;
          tax=0;
        end;

        pfkeep=pfkeep-debt_payment-tax; 
 
        pfreplace0=trd*erentrev(0,ms,month)-trc-cmaint(0,ms,month);
        transcost=oecsales(a,ms,month)-oecsales(0,ms,month);

        if (debt_financing);
           pfreplace=transcost+pfreplace0+debt_financing*purchase_price; % amount borrowed for new machine is a cash inflow
           if (a <= loan_duration);
            pfreplace=pfreplace-debt_financing*purchase_price*ipf(a,1);  % pay off remaining loan balance on machine that was sold
           end;
        else;
           pfreplace=transcost+pfreplace0;  % value of replacing with a new machine
        end;

        if (~notax);
          if (~nodepreciation);
             tax=tax_rate*pfreplace0;
          else;
             tax=tax_rate*(pfreplace0+transcost);
          end;
        else;
          tax=0;
        end;

        pfreplace=pfreplace-tax;

        sp=salep(a,ms,month);
        %sp=(1-(dr_notax(a,ms,month)>0));
        sind=nm*na*(month-1)+nm*(a-1)+ms;
        pf(sind)=sp*pfreplace+(1-sp)*pfkeep;
        nind=(monthp-1)*na*nm+(ap-1)*nm+1;
        nind1=nind+nm-1;
        p(sind,nind:nind1)=(1-sp)*tpm(ms,:);
        nind=(monthp-1)*na*nm+1;
        nind1=nind+nm-1;
        p(sind,nind:nind1)=sp*tpm(ms,:);

    end;

  end;

end;

vsq = (eye(n)-bet*p)\pf;             % Policy valuation step
