% policy_valuation_single_cycle.m: policy valuation for the firms' status quo replacement policies
%                                  differs from policy_valuation.m by computing the value of the current
%                                  machine only, and not the present value of profits from all future machines
%
%                                  Copyright, all rights reserved John Rust, Georgetown University, John McClelland June 2018
%
% Outputs:
%
%    vsq:  value of the status quo policy (predicted by the firm's replacement policy given
%          by salep.m)
%

function [vsq,pf]=policy_valuation_single_cycle;

global vl d p n na nm bet oec trc trd rr mtp_pc pr;
global notax accounting_life loan_duration borrowing_rate tax_rate debt_financing interest_deductible ipf book_value nodepreciation;
global erms purchase_price resale_price;

pf=zeros(n,1);
mstp=zeros(nm,nm);
p=0*p;

for month=1:12;       % loop over month of the year

  i1=nm*(month-1)+1;
  i2=i1+nm-1;
  tpm=mtp_pc(i1:i2,:);        
  monthp=nextmonth(month);

  for a=1:na;         % loop over machine ages

    if (a == na);
       ap=na;
    else;
       ap=a+1;
    end;

    for ms=1:nm;      % loop over macro state

        sp=salep(a,ms,month);
        sind=nm*na*(month-1)+nm*(a-1)+ms;

        interest_payment=0;
        debt_payment=0;
        depr=0;
        tax=0;

        pfkeep=erentrev(a,ms,month)-cmaint(a,ms,month);

        if (debt_financing);
          if (a > loan_duration);    
            interest_payment=0;
            debt_payment=0;
          else;
            interest_payment=debt_financing*purchase_price*ipf(a,3);
            debt_payment=debt_financing*purchase_price*ipf(a,2);
          end;
        else;
            interest_payment=0;
            debt_payment=0;
        end;

        if (~notax);
          if (~nodepreciation);
              if (a == 1);
                depr=(purchase_price-resale_price)*(1-book_value(a));
              elseif (a < accounting_life);
                depr=(purchase_price-resale_price)*(book_value(a-1)-book_value(a));
              else;
                depr=0;
              end;
          end;

          % calculate EBT (Earnings before taxes)

          if (interest_deductible);
            accounting_profit=pfkeep-interest_payment-depr;
          else;
            accounting_profit=pfkeep-depr;
          end;
          % compute tax
          tax=tax_rate*accounting_profit;
        else;
          tax=0;
        end;

        pfkeep=pfkeep-debt_payment-tax; 

        pf(sind)=(1-sp)*pfkeep;

        nind=(monthp-1)*na*nm+(ap-1)*nm+1;
        nind1=nind+nm-1;
        p(sind,nind:nind1)=(1-sp)*tpm(ms,:);

    end;

  end;

end;

vsq = (eye(n)-bet*p)\pf;             % Policy valuation step
