% setup.m:  setup file for parameters for the DP solution, dpsolve.m
%           Copyright, all rights reserved, John Rust, Georgetown University,  John McClelland, ARA, June 2018

global v vl vsq d p n na nm bet trc trd rr tol maxit pr erms fc;

% tax policy parameters

global notax tax_regime accounting_life borrowing_rate loan_duration tax_rate debt_financing interest_deductible ipf book_value nodepreciation;

tax_regime='sq';      % set this as a label for the tax regime ('sq' for current 35% tax or 'trump' for 20% tax, with nodeprecation=1
                      % and interest_deductible=0 

notax=1;              % set this to 1 if you do not want to solve the model
                      % accounting for the tax parameters below
nodepreciation=0;     % set this to 1 if machines are expensed 100% when acquired
                      % (this also implies that the sale price of the machine is taxed
                      % as revenue)
debt_financing=0.0;   % set this to a number between 0 and 1 to calculate discounted cash flows assuming 
                      % x% debt financing of a new machine with a loan at the borrowing_rate (see below) 
                      % over a loan duration equal to the accounting life below.
interest_deductible=0;% set to 1 if interest is a tax-deductible expense

accounting_life=6*12; % assumed lifespan of a machine for accounting depreciation (book value) purposes (months)
loan_duration=6*12;   % duration of a loan (in months) to finance the purchase of the new machine
borrowing_rate=.05;   % the rate at which the firm can borrow to finance the acquisition cost of a machine if debt_financing=1
tax_rate=.35;         % the corporate tax rate

if (strcmp(tax_regime,'trump'));
   tax_rate=.20;
   nodepreciation=1;
   interest_deductible=1;
end;

ipf=interest(loan_duration,borrowing_rate);           % precompute a matrix showing the remaining balance and interest/principal 
                                                      % payments on a loan to finance the machine
book_value=depreciation_schedule(accounting_life);    % this precalculated the book value of the machine over its accounting life
                                                      % under the MACRs depreciation rule (unless nodepreciation=1, in which case
                                                      % accounting depreciation is ignored and the cost of the machine is immediately
                                                      % expensed at acquisition date
% model parameters

% the macro states are discretized into n categories, 1,...,n with the lowest indicating
% periods with the lowest aggregate construction spending and the highest being the periods with the highest
% aggregate construction spending which are interpreted as a ``macro shock'' that has persistent effects on the
% demand for rental equipment.  For nm=3, then m=3 is a  "boom" period and m=1 is a "recession" period and m=2 can
% be interpreted as a period of "normal" construction demand

capcost=0.07;% opportunity cost of capital for the firm, expressed as an annual interest rate
bet=exp(-capcost/12);  % monthly discount factor 
trc=1000;              % transaction cost
fc=0;                  % fixed monthly costs per machine (i.e. costs that do not depend on the age of the machine
                       % and capture other costs such as insurance, marketing, delivery costs, and so forth that are
                       % not captured by our data on maintenance costs
trd=0.5;               % transaction disruption: a fraction between 0 and 1 providing the fraction of revenues
                       % earned by a new machine in the month it is replaced. 
nm=3;                  % number of macro shocks (index of construction spending)
na=250;      % number of possible ages for the machine (monthly intervals, so divide by 12 to get maximum age of machine in years)
rr=1;        % restrict replacement policy: if 1, can only replace existing machine with new one,
             %                              if 0, can replace an existing machine with a used machine that is younger than the current one
             %                              if < 0, can replace an existing machine with any machine, even if the replacement is older than
             %                                      the current machine, except that machines of age at most -rr are allowed as valid
             %                                      replacement machines

% algorithm parameters

pr=0;                  % print flag: set to 1 for information on the policy_improvement calculation, or 0 for no debugging output
tol=10^(-8);           % convergence tolerance for policy iteration algorithm
maxit=25;              % maximum number of iterations of policy iteration algorithm

%  do not change this file below here


n=na*nm*12;  % total dimension of problem, number of discrete states


v=zeros(n,1);    % value function, stores the final solution
vl=zeros(n,1);   % temporary space for value function in successive approximation iterations
d=zeros(n,1);    % decision rule: optimal age of machine to hold
p=zeros(n,n);    % markov transition probability
